#include <gtk/gtk.h>
#include "guiutils.h"
#include "rootwin.h"


void iv_put_image_to_root_centered(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format	/* Any of IMGVIEW_FORMAT_*. */
);
void iv_put_image_to_root_tofit(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format	/* Any of IMGVIEW_FORMAT_*. */
);
void iv_put_image_to_root_tocover(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format     /* Any of IMGVIEW_FORMAT_*. */
);
void iv_put_image_to_root_tiled(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format     /* Any of IMGVIEW_FORMAT_*. */
);


/*
 *	Put given image data to the root window centered.
 */
void iv_put_image_to_root_centered(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format
)
{
        gint rx, ry, rwidth, rheight, rdepth;
        GdkPixmap *pixmap;


        if((root_window == NULL) || (data == NULL) || (gc == NULL) ||
           (width < 1) || (height < 1) || (bpl <= 0) || (bpp <= 0)
        )
            return;

        /* Get size of root window. */
        gdk_window_get_geometry(
            root_window, &rx, &ry, &rwidth, &rheight, &rdepth
        );
        /* Root window size valid? */
        if((rwidth < 1) || (rheight < 1))
            return;

        /* Create tempory GdkPixmap. */
        pixmap = gdk_pixmap_new(root_window, rwidth, rheight, rdepth);
        if(pixmap != NULL)
        {
            /* Put given data to the GdkPixmap. */
            switch(bpp)
            {
              case 4:
                gdk_draw_rgb_32_image(
                    (GdkDrawable *)pixmap, gc,
                    (rwidth / 2) - (width / 2),
		    (rheight / 2) - (height / 2),
		    width, height,
                    GDK_RGB_DITHER_NORMAL,
                    (guchar *)data,
                    bpl
                );
                break;

              case 3:
                gdk_draw_rgb_image(
                    (GdkDrawable *)pixmap, gc,
                    (rwidth / 2) - (width / 2),
                    (rheight / 2) - (height / 2),
                    width, height,
                    GDK_RGB_DITHER_NORMAL,
                    (guchar *)data,
                    bpl
                );
                break;
            }

            /* Set this pixmap to the root window. */
            gdk_window_set_back_pixmap(
                root_window, pixmap, FALSE
            );
            /* Redraw the root window. */
            gdk_window_clear(root_window);

            /* Unref the tempory GdkPixmap, the root window should keep
             * one refcount that it will manage.
             */
            gdk_pixmap_unref(pixmap);
            pixmap = NULL;
        }
}

/*
 *	Put given image data to the root window to fit, keeping
 *	original aspect.
 */
void iv_put_image_to_root_tofit(
	GdkWindow *root_window, GdkGC *gc,
	const guint8 *data,
	gint width, gint height, gint bpl, gint bpp,
	gint format
)
{
	guint8 *tar_data;
	gdouble itor_zoom;
	gint rx, ry, rwidth, rheight, rdepth;
	gint twidth, theight, tbpl, tbpp;
	GdkPixmap *pixmap;


	if((root_window == NULL) || (data == NULL) || (gc == NULL) ||
           (width < 1) || (height < 1) || (bpl <= 0) || (bpp <= 0)
	)
	    return;

	/* Get size of root window. */
	gdk_window_get_geometry(
	    root_window, &rx, &ry, &rwidth, &rheight, &rdepth
	);
	/* Root window size valid? */
	if((rwidth < 1) || (rheight < 1))
	    return;

	/* Calculate image to root window zoom coefficient. */
	itor_zoom = (gdouble)rwidth / (gdouble)width;
	if((itor_zoom * height) > rheight)
	    itor_zoom = (gdouble)rheight / (gdouble)height;

	/* Calculate target image geometry. */
	twidth = itor_zoom * width;
	theight = itor_zoom * height;

	/* Allocate target image data. */
	tbpp = bpp;
	tbpl = twidth * tbpp;
	tar_data = (guint8 *)g_malloc(tbpl * theight);
	if(tar_data == NULL)
	    return;

	/* Copy source data to target data and resize. */
	GUIResizeBuffer(
	    bpp,
	    (const gpointer)data, width, height, bpl,
	    (gpointer)tar_data, twidth, theight, tbpl
	);

	/* Create tempory GdkPixmap the exact size to match the root
	 * window.
	 */
	pixmap = gdk_pixmap_new(root_window, rwidth, rheight, rdepth);
	if(pixmap != NULL)
	{
	    /* Put target data to pixmap. */
	    switch(bpp)
	    {
	      case 4:
		gdk_draw_rgb_32_image(
		    (GdkDrawable *)pixmap, gc,
		    (rwidth / 2) - (twidth / 2),
		    (rheight / 2) - (theight / 2),
		    twidth, theight,
		    GDK_RGB_DITHER_NORMAL,
		    tar_data,
		    tbpl
		);
		break;

              case 3:
                gdk_draw_rgb_image(
                    (GdkDrawable *)pixmap, gc,
                    (rwidth / 2) - (twidth / 2),
                    (rheight / 2) - (theight / 2),
                    twidth, theight,
                    GDK_RGB_DITHER_NORMAL,
                    tar_data,
                    tbpl
                );
                break;
	    }

	    /* Set this pixmap to the root window. */
	    gdk_window_set_back_pixmap(
		root_window, pixmap, FALSE
	    );
	    /* Redraw the root window. */
	    gdk_window_clear(root_window);

	    /* Unref the tempory GdkPixmap, the root window should keep
	     * one refcount that it will manage.
	     */
	    gdk_pixmap_unref(pixmap);
	    pixmap = NULL;
	}

	/* Deallocate data for root window, it is no longer needed. */
	g_free(tar_data);
	tar_data = NULL;
}

/*
 *	Put given image data to the root window to cover it completly,
 *	keeping original aspect but may obscure some parts of the image.
 */
void iv_put_image_to_root_tocover(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format     /* Any of IMGVIEW_FORMAT_*. */
)
{
        guint8 *tar_data;
        gdouble itor_zoom;
        gint rx, ry, rwidth, rheight, rdepth;
        gint twidth, theight, tbpl, tbpp;
        GdkPixmap *pixmap;


        if((root_window == NULL) || (data == NULL) || (gc == NULL) ||
           (width < 1) || (height < 1) || (bpl <= 0) || (bpp <= 0)
        )
            return;

        /* Get size of root window. */
        gdk_window_get_geometry(
            root_window, &rx, &ry, &rwidth, &rheight, &rdepth
        );
        /* Root window size valid? */
        if((rwidth < 1) || (rheight < 1))
            return;

        /* Calculate image to root window zoom coefficient. */
        itor_zoom = (gdouble)rwidth / (gdouble)width;
        if((itor_zoom * height) < rheight)
            itor_zoom = (gdouble)rheight / (gdouble)height;

        /* Calculate target image geometry. */
        twidth = itor_zoom * width;
        theight = itor_zoom * height;

        /* Allocate target image data. */
        tbpp = bpp;
        tbpl = twidth * tbpp;
        tar_data = (guint8 *)g_malloc(tbpl * theight);
        if(tar_data == NULL)
            return;

        /* Copy source data to target data and resize. */
        GUIResizeBuffer(
            bpp,
            (const gpointer)data, width, height, bpl,
            (gpointer)tar_data, twidth, theight, tbpl
        );

        /* Create tempory GdkPixmap the exact size to match the root
         * window.
         */
        pixmap = gdk_pixmap_new(root_window, rwidth, rheight, rdepth);
        if(pixmap != NULL)
        {
            /* Put target data to pixmap. */
            switch(bpp)
            {
              case 4:
                gdk_draw_rgb_32_image(
                    (GdkDrawable *)pixmap, gc,
                    (rwidth / 2) - (twidth / 2),
                    (rheight / 2) - (theight / 2),
                    twidth, theight,
                    GDK_RGB_DITHER_NORMAL,
                    tar_data,
                    tbpl
                );
                break;

              case 3:
                gdk_draw_rgb_image(
                    (GdkDrawable *)pixmap, gc,
                    (rwidth / 2) - (twidth / 2),
                    (rheight / 2) - (theight / 2),
                    twidth, theight,
                    GDK_RGB_DITHER_NORMAL,
                    tar_data,
                    tbpl
                );
                break;
            }

            /* Set this pixmap to the root window. */
            gdk_window_set_back_pixmap(
                root_window, pixmap, FALSE
            );
            /* Redraw the root window. */
            gdk_window_clear(root_window);

            /* Unref the tempory GdkPixmap, the root window should keep
             * one refcount that it will manage.
             */
            gdk_pixmap_unref(pixmap);
            pixmap = NULL;
        }

        /* Deallocate data for root window, it is no longer needed. */
        g_free(tar_data);
        tar_data = NULL;
}

/*
 *      Put given image data to the root window tiled.
 */
void iv_put_image_to_root_tiled(
        GdkWindow *root_window, GdkGC *gc,
        const guint8 *data,
        gint width, gint height, gint bpl, gint bpp,
        gint format
)
{
        gint rx, ry, rwidth, rheight, rdepth;
        GdkPixmap *pixmap;


        if((root_window == NULL) || (data == NULL) || (gc == NULL) ||
           (width < 1) || (height < 1) || (bpl <= 0) || (bpp <= 0)
        )
            return;

        /* Get size of root window. */
        gdk_window_get_geometry(
            root_window, &rx, &ry, &rwidth, &rheight, &rdepth
        );
        /* Root window size valid? */
        if((rwidth < 1) || (rheight < 1))
            return;

        /* Create tempory GdkPixmap. */
        pixmap = gdk_pixmap_new(root_window, rwidth, rheight, rdepth);
        if(pixmap != NULL)
        {
	    gint x, y, x_start, y_start;

            x_start = (((rwidth / 2) - (width / 2)) % width) - width;
	    y_start = (((rheight / 2) - (height / 2)) % height) - height;

	    /* Itterate through tiles. */
	    for(y = y_start; y < rheight; y += height)
	    {
		for(x = x_start; x < rwidth; x += width)
		{
		    /* Put given image to tiled position. */
		    switch(bpp)
		    {
		      case 4:
			gdk_draw_rgb_32_image(
			    (GdkDrawable *)pixmap, gc,
			    x, y, width, height,
			    GDK_RGB_DITHER_NORMAL,
			    (guchar *)data,
			    bpl
			);
			break;

		      case 3:
			gdk_draw_rgb_image(
			    (GdkDrawable *)pixmap, gc,
			    x, y, width, height,
			    GDK_RGB_DITHER_NORMAL,
			    (guchar *)data,
			    bpl
			);
			break;
		    }
		}
	    }

            /* Set this pixmap to the root window. */
            gdk_window_set_back_pixmap(
                root_window, pixmap, FALSE
            );
            /* Redraw the root window. */
            gdk_window_clear(root_window);

            /* Unref the tempory GdkPixmap, the root window should keep
             * one refcount that it will manage.
             */
            gdk_pixmap_unref(pixmap);
            pixmap = NULL;
        }
}
